#!/usr/bin/perl

use strict;
use CGI::Carp qw(fatalsToBrowser);
use CGI;
use Encode qw(decode_utf8);

BEGIN { require "wakautils.pl" }

use constant MAPS_KEY => ""; # Get a Google Maps API key from http://www.google.com/apis/maps/signup.html and put it here.
use constant STATIC_FILE => ""; # Set this to a local filename if you want the script to only show one specific file.
use constant MAX_DOWNLOAD_SIZE => 65536; # Limit the maximum size the script will download over HTTP here.

use constant VIEW_TEMPLATE => compile_template(q{
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">

<head>
<meta http-equiv="content-type" content="text/html; charset=UTF-8" />

<title>KML viewer</title>

<style type="text/css">

#map {
	position: absolute;
	left: 8px;
	top: 8px;
	border: 2px solid black;
	width:600px;
	height:600px;
}
body>#map {
	width: auto;
	height: auto;
	right: 264px;
	bottom: 8px;
}
#list {
	position: absolute;
	left: 616px;
	top: 8px;
	width: 230px;
	height: 584px;
	border: 2px solid #003f00;
	padding: 8px;
	background: #efffef;
	font-family: sans-serif;
	font-size: 10pt;
	overflow: auto;
}
body>#list {
	height: auto;
	left: auto;
	right: 8px;
	bottom: 8px;
}
#list h2 {
	font-size: 12pt;
	margin: 0px 0px 6px 0px;
	padding: 0px;
	color: #003f00;
	font-weight: bold;
}
#list a {
	display:block;
	color: #003f00;
	text-decoration: none;
	margin-bottom: 2px;
}
#list a:hover {
	color: #007f00;
}
.bubble {
	width: 300px;
	font-size: 10pt;
}
.bubble strong {
	font-family: sans-serif;
}
</style>

<script src="http://maps.google.com/maps?file=api&amp;v=1&amp;key=<const MAPS_KEY>" type="text/javascript">
</script>

<script type="text/javascript">
//<![CDATA[

function init()
{
	if(!GBrowserIsCompatible()) return;

	var map=new GMap(document.getElementById("map"));

	map.addControl(new GLargeMapControl());
	map.addControl(new GMapTypeControl());

	resize_map(map,<var $minlong>,<var $maxlong>,<var $minlat>,<var $maxlat>);

	var marker;
	<loop $places>
	map.addOverlay(make_marker(map,<var $long>,<var $lat>,<var js_string($name)>,<var js_string($description)>));
	</loop>
}

function make_marker(map,lo,la,n,desc)
{
	var point=new GPoint(lo,la);
	var marker=new GMarker(point);

	var html='<div class="bubble">';
	html+='<strong>'+n+'</strong>';
	if(desc) html+='<p>'+desc+'</p>';
	html+='</div>';

	GEvent.addListener(marker,'click',function() {
		marker.openInfoWindowHtml(html);
	});

	var list=document.getElementById("list");
	var link=document.createElement("a");
	link.innerHTML=n;
	link.href="javascript:void(0)";
	link.onclick=function() {
		map.recenterOrPanToLatLng(point);
		marker.openInfoWindowHtml(html);
	}

	list.appendChild(link);

	return marker;
}

function resize_map(map,minlong,maxlong,minlat,maxlat)
{
	var center=new GPoint((maxlong+minlong)/2,(maxlat+minlat)/2);

	for(var i=0;i<16;i++)
	{
		map.centerAndZoom(center,i);
		var bounds=map.getBoundsLatLng();
		if(bounds.minX<minlong&&bounds.maxX>maxlong&&bounds.minY<minlat&&bounds.maxY>maxlat) break;
	}
}

//]]>
</script>

</head>
<body onload="init()">
<div id="map"></div>
<div id="list">
<h2>Locations</h2>
</div>
</body>
</html>

},1);

my $kml;

if(STATIC_FILE)
{
	$kml=read_file(STATIC_FILE);
}
else
{
	my $query=new CGI;
	my $url=$query->param("url");
	my $file=$query->param("file");
	my $text=$query->param("text");

	if($url) { $kml=get_http($url,MAX_DOWNLOAD_SIZE,"","kmlcache/cache-") }
	elsif($file) { $kml=read_file($file) }
	elsif($text) { $kml=$text }
}

$kml=z_unpack($kml) if $kml=~/^PK/;

my @places=parse_kml($kml);
my ($minlong,$maxlong,$minlat,$maxlat)=find_extents(@places);


print "Content-Type: text/html; charset=utf-8\n";
print "\n";
print VIEW_TEMPLATE->(
	minlong=>$minlong,
	maxlong=>$maxlong,
	minlat=>$minlat,
	maxlat=>$maxlat,
	places=>\@places,
);

sub parse_kml($)
{
	my ($kml)=@_;

	$kml=decode_utf8($kml);

	return map
	{
		my ($name)=m!<name>(.*?)</name>!s;
		my ($long,$lat)=m!<coordinates>(.*?),(.*?),(.*?)</coordinates>!s;
		my ($description)=m!<description>(.*?)</description>!s;

		{ name=>$name, long=>$long, lat=>$lat, description=>$description }
	} $kml=~m!<Placemark>(.*?)</Placemark>!gs;
}

sub find_extents(@)
{
	my ($minlong,$maxlong,$minlat,$maxlat)=(1000,-1000,1000,-1000);
	foreach my $place (@_)
	{
		$minlong=$$place{long} if $$place{long}<$minlong;
		$maxlong=$$place{long} if $$place{long}>$maxlong;
		$minlat=$$place{lat} if $$place{lat}<$minlat;
		$maxlat=$$place{lat} if $$place{lat}>$maxlat;
	}
	return ($minlong,$maxlong,$minlat,$maxlat);
}

sub read_file($)
{
	my $file=shift;
	my ($filehandle,$contents,$buffer);

	if(ref $file eq "GLOB") { $filehandle=$file }
	else { open $filehandle,$file or return undef }

	binmode $filehandle;
	while(read $filehandle,$buffer,4096) { $contents.=$buffer }

	return $contents;
}

sub z_unpack($)
{
	my $file=shift;
	my $name="/tmp/kmlview".make_random_string(10);

	open ZCAT,"|zcat >$name" or die "Couldn't run zcat to decompress file";
	print ZCAT $file;
	close ZCAT;

	my $decompressed=read_file($name);

	unlink $file;

	return $decompressed;
}
